
// This code controls 4 simultaneous steppers
// It reads data from the serial monitor, that is sent through Bluetooth, 
// and controls the steppers accordingly
//  THis code runs all steppers at the same time.
//
// Copyright (C) 2018 Wael Khalil

#include <TimerOne.h>

//include the library code
#include <AccelStepper.h> //stepper motors library

//include the library code
#include <Servo.h> //servo motor library

//Define pins used for the Servo Motor
const int ServoPin = 7;

Servo myServo; // create servo object to control a servo

//define some contant numbers that will be used in the code
const int MaxSpeedSlow = 300;
const int AccelerationSlow = 100;
const int SlowDistance = 10000;

bool stopped = true;

const int Initialize_Distance = 50;

// Define some steppers and the pins they will use
AccelStepper stepper1(1, 8, 9); 
AccelStepper stepper2(1, 5, 6); 
AccelStepper stepper3(1, 3, 2); 
AccelStepper stepper4(1, A2, 4); 


void setup()
{  
  pinMode(13,OUTPUT);
  // initialize serial:
  Serial.begin(9600);

  //Define servo pin
  myServo.attach(7);

  //initialize servo position
  myServo.write(90);
  
  delay (2000);

  //set initial stepper settings
  stepper1.setMaxSpeed(MaxSpeedSlow);
  stepper1.setAcceleration(AccelerationSlow);
  
  stepper2.setMaxSpeed(MaxSpeedSlow);
  stepper2.setAcceleration(AccelerationSlow);
  
  stepper3.setMaxSpeed(MaxSpeedSlow);
  stepper3.setAcceleration(AccelerationSlow);
  
  stepper4.setMaxSpeed(MaxSpeedSlow);
  stepper4.setAcceleration(AccelerationSlow);



 Timer1.initialize(250); // set a timer of length 100000 microseconds (or 0.1 sec - or 10Hz => the led will blink 5 times, 5 cycles of on-and-off, per second)
  Timer1.attachInterrupt( doSteps ); // attach the service routine here
  
  //Initialize Robot
  stepper1.moveTo(-Initialize_Distance);
  stepper2.moveTo(Initialize_Distance);
  stepper3.moveTo(Initialize_Distance);
  stepper4.moveTo(-Initialize_Distance);
 
  stepper1.runToPosition();
  stepper2.runToPosition();
  stepper3.runToPosition();
  stepper4.runToPosition();
 
  delay(5000);


}

  void doSteps()
{
  if(stopped == false)
  {
    stepper1.runSpeed();
    stepper2.runSpeed();
    stepper3.runSpeed();
    stepper4.runSpeed();
  }
    // Toggle LED
    digitalWrite( 13, digitalRead( 13 ) ^ 1 );
}




void forward() 
{
  //set stepper settings
  stepper1.setSpeed(-MaxSpeedSlow);
  
  stepper2.setSpeed(MaxSpeedSlow);
  
  stepper3.setSpeed(MaxSpeedSlow);
  
  stepper4.setSpeed(-MaxSpeedSlow);
}

void back() 
{
  //set stepper settings
  stepper1.setSpeed(MaxSpeedSlow);
  
  stepper2.setSpeed(-MaxSpeedSlow);
  
  stepper3.setSpeed(-MaxSpeedSlow);
  
  stepper4.setSpeed(MaxSpeedSlow);
}


void left() 
{
  //set stepper settings
  stepper1.setSpeed(-MaxSpeedSlow);
  
  stepper2.setSpeed(-MaxSpeedSlow);
  
  stepper3.setSpeed(MaxSpeedSlow);
  
  stepper4.setSpeed(MaxSpeedSlow);  
}



void right() 
{
  //set stepper settings
  stepper1.setSpeed(MaxSpeedSlow);
  
  stepper2.setSpeed(MaxSpeedSlow);
  
  stepper3.setSpeed(-MaxSpeedSlow);
  
  stepper4.setSpeed(-MaxSpeedSlow);  
}



void forwardleft() 
{
  //set stepper settings
  stepper1.setSpeed(-MaxSpeedSlow);
  
  stepper2.setSpeed(0);
  
  stepper3.setSpeed(MaxSpeedSlow);
  
  stepper4.setSpeed(0);  
}



void forwardright() 
{
  //set stepper settings
  stepper1.setSpeed(0);
  
  stepper2.setSpeed(MaxSpeedSlow);
  
  stepper3.setSpeed(0);
  
  stepper4.setSpeed(-MaxSpeedSlow);  
}



void backright() 
{
  //set stepper settings
  stepper1.setSpeed(MaxSpeedSlow);
  
  stepper2.setSpeed(0);
  
  stepper3.setSpeed(-MaxSpeedSlow);
  
  stepper4.setSpeed(0);  
}



void backleft() 
{
 //set stepper settings
  stepper1.setSpeed(0);
  
  stepper2.setSpeed(-MaxSpeedSlow);
  
  stepper3.setSpeed(0);
  
  stepper4.setSpeed(MaxSpeedSlow);
}



void rotateLeft() 
{
 //set stepper settings
  stepper1.setSpeed(MaxSpeedSlow);
  
  stepper2.setSpeed(MaxSpeedSlow);
  
  stepper3.setSpeed(MaxSpeedSlow);
  
  stepper4.setSpeed(MaxSpeedSlow);  
}

void rotateRight() 
{
 //set stepper settings
  stepper1.setSpeed(-MaxSpeedSlow);
  
  stepper2.setSpeed(-MaxSpeedSlow);
  
  stepper3.setSpeed(-MaxSpeedSlow);
  
  stepper4.setSpeed(-MaxSpeedSlow); 
}

void Stop() 
{
  stepper1.stop();
  stepper2.stop();
  stepper3.stop();
  stepper4.stop();
}


void loop()
{
  while (Serial.available())
  {
    char inChar = (char)Serial.read();
    
    switch(inChar) 
    {
      case 'F':
      stopped = false;
        forward();
        break;
       
      case 'G':
      stopped = false;
        back();
        break;
       
      case 'L':
      stopped = false;
        left();
        break;
       
      case 'R':
      stopped = false;
        right();
        break;
      
      case 'Q':
      stopped = false;
        forwardleft();
        break;
      
      case 'E':
      stopped = false;
        forwardright();
        break;
     
      case 'Z':
      stopped = false;
        backleft();
        break;
      
      case 'C':
      stopped = false;
        backright();
        break;

        case 'X':
      stopped = false;
        rotateLeft();
        break;
      
      case 'Y':
      stopped = false;
        rotateRight();
        break;
              
       case 'S':
       stopped=true;
       Stop();
       break; 
    }
  }
}





